# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Postback(UniversalBaseModel):
    action_type: typing.Literal["postback"] = pydantic.Field(default="postback")
    """
    The type of action being sent
    """

    payload: str = pydantic.Field()
    """
    The postback payload. Maximum length is 1000 characters.
    """

    execute: typing.Optional[str] = pydantic.Field(default=None)
    """
    The action to execute. Optional additional data sent when the button is tapped.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
