# This file was auto-generated by Fern from our API Definition.

import typing
from .sms import Sms
from .basic_rcs import BasicRcs
from .media_rcs import MediaRcs
from .card_rcs import CardRcs
from .carousel_rcs import CarouselRcs

SendRequest = typing.Union[Sms, BasicRcs, MediaRcs, CardRcs, CarouselRcs]
