# This file was auto-generated by Fern from our API Definition.

import typing
from .url import Url
from .call import Call
from .postback import Postback
from .share_location import ShareLocation
from .view_location import ViewLocation

Action = typing.Union[Url, Call, Postback, ShareLocation, ViewLocation]
