# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .phone_number import PhoneNumber
import pydantic
from .media_rcs_message import MediaRcsMessage
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class MediaRcs(UniversalBaseModel):
    phone_number: typing.Optional[PhoneNumber] = pydantic.Field(default=None)
    """
    Phone number to send the SMS message to
    """

    message_type: typing.Literal["media"] = pydantic.Field(default="media")
    """
    The type of message being sent
    """

    message: MediaRcsMessage = pydantic.Field()
    """
    The content of the message
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
