# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing_extensions
import typing
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SmsMessage(UniversalBaseModel):
    """
    The content of the message
    """

    body: str = pydantic.Field()
    """
    The body of the SMS message
    """

    media_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="mediaUrl")] = pydantic.Field(
        default=None
    )
    """
    Optional media URL for the SMS message
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
