# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .card import Card
import pydantic
from .action import Action
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CardRcsMessage(UniversalBaseModel):
    """
    The content of the message
    """

    cards: typing.List[Card] = pydantic.Field()
    """
    Array of one containing the card to be sent
    """

    quick_replies: typing.Optional[typing.List[Action]] = pydantic.Field(default=None)
    """
    Array of quick replies attached to the card. Maximum of 11 quick replies.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
