# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class LatLng(UniversalBaseModel):
    """
    The latitude and longitude of the location to share with the user
    """

    lat: float = pydantic.Field()
    """
    The latitude of the location
    """

    lng: float = pydantic.Field()
    """
    The longitude of the location
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
